class Player {
  PVector position,velocity; // PVector contains two floats, x and y

  Boolean isOnGround; // used to keep track of whether the player is on the ground. useful for control and animation.
  Boolean facingRight; // used to keep track of which direction the player last moved in. used to flip player image.
  int animDelay; // countdown timer between animation updates
  int animFrame; // keeps track of which animation frame is currently shown for the player
  int coinsCollected; // a counter to keep a tally on how many coins the player has collected
  
  static final float JUMP_POWER = 10.0; // how hard the player jolts upward on jump
  static final float RUN_SPEED = 4.0; // force of player movement on ground, in pixels/cycle
  static final float AIR_RUN_SPEED = 1.; // like run speed, but used for control while in the air
  static final float SLOWDOWN_PERC = 0.6; // friction from the ground. multiplied by the x speed each frame.
  static final float AIR_SLOWDOWN_PERC = .85; // resistance in the air, otherwise air control enables crazy speeds
  static final int RUN_ANIMATION_DELAY = 4; // how many game cycles pass between animation updates?
  static final float TRIVIAL_SPEED = 1.0; // if under this speed, the player is drawn as standing still
  
  Player() {
    isOnGround = false;
    facingRight = true;
    position = new PVector();
    velocity = new PVector();
    reset();
  }
  
  void reset() {
    animDelay = 0;
    animFrame = 0;
    velocity.x = 0;
    velocity.y = 0;
    coinsCollected = 0;
  }
  
  void inputCheck() {
    // keyboard flags are set by keyPressed/keyReleased in the main .pde
    
    float speedHere = (isOnGround ? RUN_SPEED : AIR_RUN_SPEED);
    float frictionHere = (isOnGround ? SLOWDOWN_PERC : AIR_SLOWDOWN_PERC);
    
    if(holdingLeft) {
      velocity.x -= speedHere;
    } else if(holdingRight) {
      velocity.x += speedHere;
    } 
    velocity.x *= frictionHere; // causes player to constantly lose speed
    
    if(isOnGround) { // player can only jump if currently on the ground
      if(holdingSpace || holdingUp) { // either up arrow or space bar cause the player to jump
        sndJump.trigger(); // play sound
        velocity.y = -JUMP_POWER; // adjust vertical speed
        isOnGround = false; // mark that the player has left the ground, i.e. cannot jump again for now
      }
    }
  }
  
  void checkForWallBumping() {
    int guyWidth = characterStart.width; // think of image size of player standing as the player's physical size
    int guyHeight = characterStart.height;
    int wallProbeDistance = int(guyWidth*0.5);
    int ceilingProbeDistance = int(guyHeight);   
    
    // used as probes to detect running into walls, ceiling
    PVector leftSideHigh,rightSideHigh,leftSideLow,rightSideLow,topSide;
    leftSideHigh = new PVector();
    rightSideHigh = new PVector();
    leftSideLow = new PVector();
    rightSideLow = new PVector();
    topSide = new PVector();

    // update wall probes
    leftSideHigh.x = leftSideLow.x = position.x - wallProbeDistance; // left edge of player
    rightSideHigh.x = rightSideLow.x = position.x + wallProbeDistance; // right edge of player
    leftSideLow.y = rightSideLow.y = position.y-0.2*guyHeight; // shin high
    leftSideHigh.y = rightSideHigh.y = position.y-0.8*guyHeight; // shoulder high

    topSide.x = position.x; // center of player
    topSide.y = position.y-ceilingProbeDistance; // top of guy
   
   //test wall probe locations/view wall probes 
//    rect(topSide.x,topSide.y,5,5);
//    rect(rightSideHigh.x,rightSideHigh.y,5,5);
//    rect(rightSideLow.x,rightSideLow.y,5,5);
//    rect(leftSideHigh.x,leftSideHigh.y,5,5);
//    rect(leftSideLow.x,leftSideLow.y,5,5);
    
    // if any edge of the player is inside a red killblock, reset the round
    if( theWorld.worldSquareAt(topSide)==World.TILE_KILLBLOCK ||
         theWorld.worldSquareAt(leftSideHigh)==World.TILE_KILLBLOCK ||
         theWorld.worldSquareAt(leftSideLow)==World.TILE_KILLBLOCK ||
         theWorld.worldSquareAt(rightSideHigh)==World.TILE_KILLBLOCK ||
         theWorld.worldSquareAt(rightSideLow)==World.TILE_KILLBLOCK ||
         theWorld.worldSquareAt(position)==World.TILE_KILLBLOCK) {
      resetGame();
      lives--;
      life.trigger();
      return; // any other possible collisions would be irrelevant, exit function now
    }
    
    // the following conditionals just check for collisions with each bump probe
    // depending upon which probe has collided, we push the player back the opposite direction
    
    if( theWorld.worldSquareAt(topSide)==World.TILE_SOLID||
        theWorld.worldSquareAt(topSide)==World.TILE_SOLIDTHIN||
        theWorld.worldSquareAt(topSide)==World.TILE_EDGEL||
        theWorld.worldSquareAt(topSide)==World.TILE_EDGER) {
      if(theWorld.worldSquareAt(position)==World.TILE_SOLID||
         theWorld.worldSquareAt(position)==World.TILE_SOLIDTHIN||
         theWorld.worldSquareAt(position)==World.TILE_EDGEL||
         theWorld.worldSquareAt(position)==World.TILE_EDGER) {
        position.sub(velocity);
        velocity.x=0.0;
        velocity.y=0.0;
      } else {
        position.y = theWorld.bottomOfSquare(topSide)+ceilingProbeDistance;
        if(velocity.y < 0) {
          velocity.y = 0.0;
        }
      }
    }
    
    if( theWorld.worldSquareAt(leftSideLow)==World.TILE_SOLID||
        theWorld.worldSquareAt(leftSideLow)==World.TILE_SOLIDTHIN||
        theWorld.worldSquareAt(leftSideLow)==World.TILE_EDGEL||
        theWorld.worldSquareAt(leftSideLow)==World.TILE_EDGER) {
//      position.x = theWorld.rightOfSquare(leftSideLow)+wallProbeDistance;
      if(velocity.x < 0) {
        velocity.x = 0.0;
      }
    }
   
    if( theWorld.worldSquareAt(leftSideHigh)==World.TILE_SOLID||
        theWorld.worldSquareAt(leftSideHigh)==World.TILE_SOLIDTHIN||
        theWorld.worldSquareAt(leftSideHigh)==World.TILE_EDGEL||
        theWorld.worldSquareAt(leftSideHigh)==World.TILE_EDGER) {
      position.x = theWorld.rightOfSquare(leftSideHigh)+wallProbeDistance;
      if(velocity.x < 0) {
        velocity.x = 0.0;
      }
    }
   
    if( theWorld.worldSquareAt(rightSideLow)==World.TILE_SOLID||
        theWorld.worldSquareAt(rightSideLow)==World.TILE_SOLIDTHIN||
        theWorld.worldSquareAt(rightSideLow)==World.TILE_EDGEL||
        theWorld.worldSquareAt(rightSideLow)==World.TILE_EDGER) {
//      position.x = theWorld.leftOfSquare(rightSideLow)-wallProbeDistance;
      if(velocity.x > 0) {
        velocity.x = 0.0;
      }
    }
   
    if( theWorld.worldSquareAt(rightSideHigh)==World.TILE_SOLID||
        theWorld.worldSquareAt(rightSideHigh)==World.TILE_SOLIDTHIN||
        theWorld.worldSquareAt(rightSideHigh)==World.TILE_EDGEL||
        theWorld.worldSquareAt(rightSideHigh)==World.TILE_EDGER) {
      position.x = theWorld.leftOfSquare(rightSideHigh)-wallProbeDistance;
      if(velocity.x > 0) {
        velocity.x = 0.0;
      }
    }
  }

  void checkForCoinGetting() {
    PVector centerOfPlayer;
    // we use this to check for coin overlap in center of player
    // (remember that "position" is keeping track of bottom center of feet)
    centerOfPlayer = new PVector(position.x,position.y-characterStart.height*.5);

    if(theWorld.worldSquareAt(centerOfPlayer)==World.TILE_COIN) {
      theWorld.setSquareAtToThis(centerOfPlayer, World.TILE_EMPTY);
      sndCoin.trigger();
      coinsCollected++;
    }
  }

  void checkForFalling() {
    // If we're standing on an empty or coin tile, we're not standing on anything. Fall!
    if(theWorld.worldSquareAt(position)==World.TILE_EMPTY ||
       theWorld.worldSquareAt(position)==World.TILE_COIN ||
       theWorld.worldSquareAt(position)==World.TILE_SPIKE) {
       isOnGround=false;
    }
    
    if(isOnGround==false) { // not on ground?    
      if(theWorld.worldSquareAt(position)==World.TILE_SOLID||
         theWorld.worldSquareAt(position)==World.TILE_SOLIDTHIN||
         theWorld.worldSquareAt(position)==World.TILE_EDGEL||
         theWorld.worldSquareAt(position)==World.TILE_EDGER) { // landed on solid square?
        isOnGround = true;
        position.y = theWorld.topOfSquare(position);
        velocity.y = 0.0;
      } else { // fall
        velocity.y += GRAVITY_POWER;
      }
    }
  }

  void move() {
    position.add(velocity);
    
    checkForWallBumping();  
    checkForCoinGetting();
    checkForFalling();
  }
  
  void draw() {

//draw the character at the customize screen
    
if(level == 1) {
  image(characterStart,width*.2,height*.52,characterSizeX,characterSizeY);
  image(characterLegs,width*.2,height*.52,characterSizeX,characterSizeY);
  
  layerSelect = constrain(layerSelect,0,13);
 
  blink+=2;
  if(blink == 1000) {
    blink = 0;
  }
  
  if(blink > 190 && blink < 200 || blink > 970 && blink < 980) {
  image(characterEyelids,width*.2,height*.52,characterSizeX,characterSizeY);
  }

  image(sock[sockCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(shoe[shoeCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(undies[undiesCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(pants[pantsCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(watch[watchCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(hands[handsCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(hat[hatCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(underShirt[underShirtCall],width*.2,height*.52,characterSizeX,characterSizeY);  
  image(overShirt[overShirtCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(tie[tieCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(sweaterVest[sweaterVestCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(collar[collarCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(jacket[jacketCall],width*.2,height*.52,characterSizeX,characterSizeY);
  image(outerCoat[outerCoatCall],width*.2,height*.52,characterSizeX,characterSizeY); 
  image(scarf[scarfCall],width*.2,height*.52,characterSizeX,characterSizeY); 
    }
 
// draw the character in the game 
    
if (level == 3) {   
    int guyWidth = characterStart.width;
    int guyHeight = characterStart.height;
    
    if(velocity.x<-TRIVIAL_SPEED) {
      facingRight = false;
    } else if(velocity.x>TRIVIAL_SPEED) {
      facingRight = true;
    }
    
pushMatrix(); // lets us compound/accumulate translate/scale/rotate calls, then undo them all at once
translate(position.x,position.y);
imageMode(CORNER);

if(facingRight==true) {
      scale(-1,1); // flip horizontally by scaling horizontally by -100%
    }
    
translate(-guyWidth,-guyHeight-39); // drawing images centered on character's feet

if(isOnGround==false) { // falling or jumping
      image(characterStart, 0,0,characterGameSizeX,characterGameSizeY);
      image(characterLegsRun1, 0,0,characterGameSizeX,characterGameSizeY);
      if(blink > 190 && blink < 200 || blink > 970 && blink < 980) {
          image(characterEyelids,0,0,characterGameSizeX,characterGameSizeY);
      } 
  image(sockRun1[sockCall],0,0,characterGameSizeX,characterGameSizeY);
  image(shoeRun1[shoeCall],0,0,characterGameSizeX,characterGameSizeY);
  image(undiesRun1[undiesCall],0,0,characterGameSizeX,characterGameSizeY);
  image(pantsRun1[pantsCall],0,0,characterGameSizeX,characterGameSizeY);
  image(watch[watchCall],0,0,characterGameSizeX,characterGameSizeY);
  image(underShirt[underShirtCall],0,0,characterGameSizeX,characterGameSizeY);
  image(hands[handsCall],0,0,characterGameSizeX,characterGameSizeY);
  image(hat[hatCall],0,0,characterGameSizeX,characterGameSizeY);
  image(overShirt[overShirtCall],0,0,characterGameSizeX,characterGameSizeY);
  image(tie[tieCall],0,0,characterGameSizeX,characterGameSizeY);
  image(sweaterVest[sweaterVestCall],0,0,characterGameSizeX,characterGameSizeY);
  image(collar[collarCall],0,0,characterGameSizeX,characterGameSizeY);
  image(jacket[jacketCall],0,0,characterGameSizeX,characterGameSizeY);
  image(outerCoat[outerCoatCall],0,0,characterGameSizeX,characterGameSizeY);
  image(scarf[scarfCall],0,0,characterGameSizeX,characterGameSizeY);
    } else if(abs(velocity.x)<TRIVIAL_SPEED) { // not moving fast, i.e. standing
      image(characterStart, 0,0,characterGameSizeX,characterGameSizeY);
      image(characterLegs, 0,0,characterGameSizeX,characterGameSizeY);
        if(blink > 190 && blink < 200 || blink > 970 && blink < 980) {
          image(characterEyelids,0,0,characterGameSizeX,characterGameSizeY);
        }  
  image(sock[sockCall],0,0,characterGameSizeX,characterGameSizeY);
  image(shoe[shoeCall],0,0,characterGameSizeX,characterGameSizeY);
  image(undies[undiesCall],0,0,characterGameSizeX,characterGameSizeY);
  image(pants[pantsCall],0,0,characterGameSizeX,characterGameSizeY);
  image(watch[watchCall],0,0,characterGameSizeX,characterGameSizeY);
  image(underShirt[underShirtCall],0,0,characterGameSizeX,characterGameSizeY);
  image(hands[handsCall],0,0,characterGameSizeX,characterGameSizeY);
  image(hat[hatCall],0,0,characterGameSizeX,characterGameSizeY);
  image(overShirt[overShirtCall],0,0,characterGameSizeX,characterGameSizeY);
  image(tie[tieCall],0,0,characterGameSizeX,characterGameSizeY);
  image(sweaterVest[sweaterVestCall],0,0,characterGameSizeX,characterGameSizeY);
  image(collar[collarCall],0,0,characterGameSizeX,characterGameSizeY);
  image(jacket[jacketCall],0,0,characterGameSizeX,characterGameSizeY);
  image(outerCoat[outerCoatCall],0,0,characterGameSizeX,characterGameSizeY);
  image(scarf[scarfCall],0,0,characterGameSizeX,characterGameSizeY);
    } else { // running. Animate.
      if(animDelay--<0) {
        animDelay=RUN_ANIMATION_DELAY;
        if(animFrame==0) {
          animFrame=1;
        } else {
          animFrame=0;
        }
      }
      
if(animFrame==0) {
      image(characterStart, 0,0,characterGameSizeX,characterGameSizeY);
      image(characterLegsRun1, 0,0,characterGameSizeX,characterGameSizeY);
      if(blink > 190 && blink < 200 || blink > 970 && blink < 980) {
         image(characterEyelids,0,0,characterGameSizeX,characterGameSizeY);
      }
  image(sockRun1[sockCall],0,0,characterGameSizeX,characterGameSizeY);
  image(shoeRun1[shoeCall],0,0,characterGameSizeX,characterGameSizeY);
  image(undiesRun1[undiesCall],0,0,characterGameSizeX,characterGameSizeY);
  image(pantsRun1[pantsCall],0,0,characterGameSizeX,characterGameSizeY);
  image(watch[watchCall],0,0,characterGameSizeX,characterGameSizeY);
  image(underShirt[underShirtCall],0,0,characterGameSizeX,characterGameSizeY);
  image(hands[handsCall],0,0,characterGameSizeX,characterGameSizeY);
  image(hat[hatCall],0,0,characterGameSizeX,characterGameSizeY);
  image(overShirt[overShirtCall],0,0,characterGameSizeX,characterGameSizeY);
  image(tie[tieCall],0,0,characterGameSizeX,characterGameSizeY);
  image(sweaterVest[sweaterVestCall],0,0,characterGameSizeX,characterGameSizeY);
  image(collar[collarCall],0,0,characterGameSizeX,characterGameSizeY);
  image(jacket[jacketCall],0,0,characterGameSizeX,characterGameSizeY);
  image(outerCoat[outerCoatCall],0,0,characterGameSizeX,characterGameSizeY);
  image(scarf[scarfCall],0,0,characterGameSizeX,characterGameSizeY);
      } else {
      image(characterStart, 0,0,characterGameSizeX,characterGameSizeY);
      image(characterLegsRun2, 0,0,characterGameSizeX,characterGameSizeY);
        if(blink > 190 && blink < 200 || blink > 970 && blink < 980) {
          image(characterEyelids,0,0,characterGameSizeX,characterGameSizeY);
        }
  image(sockRun2[sockCall],0,0,characterGameSizeX,characterGameSizeY);
  image(shoeRun2[shoeCall],0,0,characterGameSizeX,characterGameSizeY);
  image(undiesRun2[undiesCall],0,0,characterGameSizeX,characterGameSizeY);
  image(pantsRun2[pantsCall],0,0,characterGameSizeX,characterGameSizeY);
  image(watch[watchCall],0,0,characterGameSizeX,characterGameSizeY);
  image(underShirt[underShirtCall],0,0,characterGameSizeX,characterGameSizeY);
  image(hands[handsCall],0,0,characterGameSizeX,characterGameSizeY);
  image(hat[hatCall],0,0,characterGameSizeX,characterGameSizeY);
  image(overShirt[overShirtCall],0,0,characterGameSizeX,characterGameSizeY);
  image(tie[tieCall],0,0,characterGameSizeX,characterGameSizeY);
  image(sweaterVest[sweaterVestCall],0,0,characterGameSizeX,characterGameSizeY);
  image(collar[collarCall],0,0,characterGameSizeX,characterGameSizeY);
  image(jacket[jacketCall],0,0,characterGameSizeX,characterGameSizeY);
  image(outerCoat[outerCoatCall],0,0,characterGameSizeX,characterGameSizeY);
  image(scarf[scarfCall],0,0,characterGameSizeX,characterGameSizeY);
      }
    }    
    popMatrix(); // undoes all translate/scale/rotate calls since the pushMatrix earlier in this function
    }
  }
}
